@@ -41,7 +41,6 @@
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/editormanager/ieditor.h>
 #include <coreplugin/messagemanager.h>
-#include <cppeditor/cppeditorconstants.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectnodes.h>
 #include <projectexplorer/projecttree.h>
@@ -97,13 +96,23 @@ public:
 
     void autoFormatOnSave(Core::IDocument *document);
 
-    QSharedPointer<GeneralSettings> m_generalSettings;
+    GeneralSettings generalSettings;
 
-    ArtisticStyle::ArtisticStyle artisticStyleBeautifier;
-    ClangFormat::ClangFormat clangFormatBeautifier;
-    Uncrustify::Uncrustify uncrustifyBeautifier;
+    ArtisticStyle artisticStyleBeautifier;
+    ClangFormat clangFormatBeautifier;
+    Uncrustify uncrustifyBeautifier;
 
-    QList<BeautifierAbstractTool *> m_tools;
+    BeautifierAbstractTool *m_tools[3] {
+        &artisticStyleBeautifier,
+        &uncrustifyBeautifier,
+        &clangFormatBeautifier
+    };
+
+    GeneralOptionsPage optionsPage {{
+        artisticStyleBeautifier.id(),
+        uncrustifyBeautifier.id(),
+        clangFormatBeautifier.id()
+    }};
 };
 
 static BeautifierPluginPrivate *dd = nullptr;
@@ -126,16 +135,7 @@ void BeautifierPlugin::extensionsInitialized()
 }
 
 BeautifierPluginPrivate::BeautifierPluginPrivate()
-    : m_tools({&artisticStyleBeautifier, &uncrustifyBeautifier, &clangFormatBeautifier})
 {
-    QStringList toolIds;
-    toolIds.reserve(m_tools.count());
-    for (BeautifierAbstractTool *tool : m_tools)
-        toolIds << tool->id();
-
-    m_generalSettings.reset(new GeneralSettings);
-    new GeneralOptionsPage(m_generalSettings, toolIds, this);
-
     updateActions();
 
     const Core::EditorManager *editorManager = Core::EditorManager::instance();
@@ -153,14 +153,14 @@ void BeautifierPluginPrivate::updateActions(Core::IEditor *editor)
 
 void BeautifierPluginPrivate::autoFormatOnSave(Core::IDocument *document)
 {
-    if (!m_generalSettings->autoFormatOnSave())
+    if (!generalSettings.autoFormatOnSave())
         return;
 
-    if (!isAutoFormatApplicable(document, m_generalSettings->autoFormatMime()))
+    if (!isAutoFormatApplicable(document, generalSettings.autoFormatMime()))
         return;
 
     // Check if file is contained in the current project (if wished)
-    if (m_generalSettings->autoFormatOnlyCurrentProject()) {
+    if (generalSettings.autoFormatOnlyCurrentProject()) {
         const ProjectExplorer::Project *pro = ProjectExplorer::ProjectTree::currentProject();
         if (!pro
             || pro->files([document](const ProjectExplorer::Node *n) {
@@ -173,10 +173,10 @@ void BeautifierPluginPrivate::autoFormatOnSave(Core::IDocument *document)
     }
 
     // Find tool to use by id and format file!
-    const QString id = m_generalSettings->autoFormatTool();
-    auto tool = std::find_if(m_tools.constBegin(), m_tools.constEnd(),
+    const QString id = generalSettings.autoFormatTool();
+    auto tool = std::find_if(std::begin(m_tools), std::end(m_tools),
                              [&id](const BeautifierAbstractTool *t){return t->id() == id;});
-    if (tool != m_tools.constEnd()) {
+    if (tool != std::end(m_tools)) {
         if (!(*tool)->isApplicable(document))
             return;
         const TextEditor::Command command = (*tool)->command();
@@ -185,7 +185,7 @@ void BeautifierPluginPrivate::autoFormatOnSave(Core::IDocument *document)
         const QList<Core::IEditor *> editors = Core::DocumentModel::editorsForDocument(document);
         if (editors.isEmpty())
             return;
-        if (auto widget = qobject_cast<TextEditorWidget *>(editors.first()->widget()))
+        if (auto widget = TextEditorWidget::fromEditor(editors.first()))
             TextEditor::formatEditor(widget, command);
     }
 }
