@@ -32,6 +32,7 @@
 #include <coreplugin/fileiconprovider.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/session.h>
+#include <texteditor/textmark.h>
 #include <utils/qtcassert.h>
 #include <utils/utilsicons.h>
 
@@ -84,13 +85,14 @@ ClangToolsDiagnosticModel::ClangToolsDiagnosticModel(QObject *parent)
     : ClangToolsDiagnosticModelBase(parent)
     , m_filesWatcher(std::make_unique<QFileSystemWatcher>())
 {
-    setHeader({tr("Diagnostic")});
+    setRootItem(new Utils::StaticTreeItem(QString()));
     connectFileWatcher();
 }
 
 QDebug operator<<(QDebug debug, const Diagnostic &d)
 {
-    return debug << "category:" << d.category
+    return debug << "name:" << d.name
+                 << "category:" << d.category
                  << "type:" << d.type
                  << "hasFixits:" << d.hasFixits
                  << "explainingSteps:" << d.explainingSteps.size()
@@ -101,13 +103,10 @@ QDebug operator<<(QDebug debug, const Diagnostic &d)
 
 void ClangToolsDiagnosticModel::addDiagnostics(const Diagnostics &diagnostics)
 {
-    const auto onFixitStatusChanged = [this](FixitStatus oldStatus, FixitStatus newStatus) {
-        if (newStatus == FixitStatus::Scheduled)
-            ++m_fixItsToApplyCount;
-        else if (oldStatus == FixitStatus::Scheduled)
-            --m_fixItsToApplyCount;
-        emit fixItsToApplyCountChanged(m_fixItsToApplyCount);
-    };
+    const auto onFixitStatusChanged =
+        [this](const QModelIndex &index, FixitStatus oldStatus, FixitStatus newStatus) {
+            emit fixitStatusChanged(index, oldStatus, newStatus);
+        };
 
     for (const Diagnostic &d : diagnostics) {
         // Check for duplicates
@@ -124,7 +123,6 @@ void ClangToolsDiagnosticModel::addDiagnostics(const Diagnostics &diagnostics)
         if (!filePathItem) {
             filePathItem = new FilePathItem(filePath);
             rootItem()->appendChild(filePathItem);
-
             addWatchedPath(d.location.filePath);
         }
 
@@ -139,12 +137,24 @@ QSet<Diagnostic> ClangToolsDiagnosticModel::diagnostics() const
     return m_diagnostics;
 }
 
+QSet<QString> ClangToolsDiagnosticModel::allChecks() const
+{
+    QSet<QString> checks;
+    forItemsAtLevel<2>([&](DiagnosticItem *item) {
+        checks.insert(item->diagnostic().name);
+    });
+
+    return checks;
+}
+
 void ClangToolsDiagnosticModel::clear()
 {
+    beginResetModel();
     m_filePathToItem.clear();
     m_diagnostics.clear();
     clearAndSetupCache();
     ClangToolsDiagnosticModelBase::clear();
+    endResetModel();
 }
 
 void ClangToolsDiagnosticModel::updateItems(const DiagnosticItem *changedItem)
@@ -208,7 +218,15 @@ static QString fixitStatus(FixitStatus status)
     return QString();
 }
 
-static QString createDiagnosticToolTipString(const Diagnostic &diagnostic, FixitStatus fixItStatus)
+static QString lineColumnString(const Debugger::DiagnosticLocation &location)
+{
+    return QString("%1:%2").arg(QString::number(location.line), QString::number(location.column));
+}
+
+static QString createDiagnosticToolTipString(
+    const Diagnostic &diagnostic,
+    Utils::optional<FixitStatus> fixItStatus = Utils::nullopt,
+    bool showSteps = true)
 {
     using StringPair = QPair<QString, QString>;
     QList<StringPair> lines;
@@ -235,9 +253,24 @@ static QString createDiagnosticToolTipString(const Diagnostic &diagnostic, Fixit
         QCoreApplication::translate("ClangTools::Diagnostic", "Location:"),
                 createFullLocationString(diagnostic.location));
 
-    lines << qMakePair(
-        QCoreApplication::translate("ClangTools::Diagnostic", "Fixit status:"),
-        fixitStatus(fixItStatus));
+    if (fixItStatus.has_value()) {
+        lines << qMakePair(QCoreApplication::translate("ClangTools::Diagnostic", "Fixit status:"),
+                           fixitStatus(fixItStatus.value()));
+    }
+
+    if (showSteps && !diagnostic.explainingSteps.isEmpty()) {
+        StringPair steps;
+        steps.first = QCoreApplication::translate("ClangTools::Diagnostic", "Steps:");
+        for (const ExplainingStep &step : diagnostic.explainingSteps) {
+            if (!steps.second.isEmpty())
+                steps.second += "<br>";
+            steps.second += QString("%1:%2: %3")
+                                .arg(step.location.filePath,
+                                     lineColumnString(step.location),
+                                     step.message);
+        }
+        lines << steps;
+    }
 
     QString html = QLatin1String("<html>"
                    "<head>"
@@ -310,11 +343,6 @@ static QString createExplainingStepString(const ExplainingStep &explainingStep,
 }
 
 
-static QString lineColumnString(const Debugger::DiagnosticLocation &location)
-{
-    return QString("%1:%2").arg(QString::number(location.line), QString::number(location.column));
-}
-
 static QString fullText(const Diagnostic &diagnostic)
 {
     QString text = diagnostic.location.filePath + QLatin1Char(':');
@@ -337,6 +365,35 @@ static QString fullText(const Diagnostic &diagnostic)
     return text;
 }
 
+static Utils::optional<QIcon> iconForType(const QString &type)
+{
+    if (type == "warning")
+        return Utils::Icons::CODEMODEL_WARNING.icon();
+    if (type == "error" || type == "fatal")
+        return Utils::Icons::CODEMODEL_ERROR.icon();
+    if (type == "note")
+        return Utils::Icons::INFO.icon();
+    if (type == "fix-it")
+        return Utils::Icons::CODEMODEL_FIXIT.icon();
+    return Utils::nullopt;
+}
+
+static TextEditor::TextMark *generateDiagnosticTextMark(const Diagnostic &diag)
+{
+    auto mark = new TextEditor::TextMark(Utils::FilePath::fromString(diag.location.filePath),
+                                         diag.location.line,
+                                         Utils::Id("ClangTool.DiagnosticMark"));
+    if (diag.type == "error" || diag.type == "fatal")
+        mark->setColor(Utils::Theme::CodeModel_Error_TextMarkColor);
+    else
+        mark->setColor(Utils::Theme::CodeModel_Warning_TextMarkColor);
+    mark->setPriority(TextEditor::TextMark::HighPriority);
+    mark->setIcon(iconForType(diag.type).value_or(Utils::Icons::CODEMODEL_WARNING.icon()));
+    mark->setToolTip(createDiagnosticToolTipString(diag));
+    mark->setLineAnnotation(diag.description);
+    return mark;
+}
+
 DiagnosticItem::DiagnosticItem(const Diagnostic &diag,
                                const OnFixitStatusChanged &onFixitStatusChanged,
                                ClangToolsDiagnosticModel *parent)
@@ -347,6 +404,8 @@ DiagnosticItem::DiagnosticItem(const Diagnostic &diag,
     if (diag.hasFixits)
         m_fixitStatus = FixitStatus::NotScheduled;
 
+    m_mark = generateDiagnosticTextMark(diag);
+
     // Don't show explaining steps if they add no information.
     if (diag.explainingSteps.count() == 1) {
         const ExplainingStep &step = diag.explainingSteps.first();
@@ -364,6 +423,7 @@ DiagnosticItem::DiagnosticItem(const Diagnostic &diag,
 DiagnosticItem::~DiagnosticItem()
 {
     setFixitOperations(ReplacementOperations());
+    delete m_mark;
 }
 
 Qt::ItemFlags DiagnosticItem::flags(int column) const
@@ -376,15 +436,8 @@ Qt::ItemFlags DiagnosticItem::flags(int column) const
 
 static QVariant iconData(const QString &type)
 {
-    if (type == "warning")
-        return Utils::Icons::CODEMODEL_WARNING.icon();
-    if (type == "error" || type == "fatal")
-        return Utils::Icons::CODEMODEL_ERROR.icon();
-    if (type == "note")
-        return Utils::Icons::INFO.icon();
-    if (type == "fix-it")
-        return Utils::Icons::CODEMODEL_FIXIT.icon();
-    return QVariant();
+    Utils::optional<QIcon> icon = iconForType(type);
+    return icon.has_value() ? QVariant(*icon) : QVariant();
 }
 
 QVariant DiagnosticItem::data(int column, int role) const
@@ -424,11 +477,13 @@ QVariant DiagnosticItem::data(int column, int role) const
             }
             break;
         }
+        case ClangToolsDiagnosticModel::DocumentationUrlRole:
+            return documentationUrl(m_diagnostic.name);
         case Qt::DisplayRole:
             return QString("%1: %2").arg(lineColumnString(m_diagnostic.location),
                                          m_diagnostic.description);
         case Qt::ToolTipRole:
-            return createDiagnosticToolTipString(m_diagnostic, m_fixitStatus);
+            return createDiagnosticToolTipString(m_diagnostic, m_fixitStatus, false);
         case Qt::DecorationRole:
             return iconData(m_diagnostic.type);
         default:
@@ -463,7 +518,11 @@ void DiagnosticItem::setFixItStatus(const FixitStatus &status)
     m_fixitStatus = status;
     update();
     if (m_onFixitStatusChanged && status != oldStatus)
-        m_onFixitStatusChanged(oldStatus, status);
+        m_onFixitStatusChanged(index(), oldStatus, status);
+    if (status == FixitStatus::Applied || status == FixitStatus::Invalidated) {
+        delete m_mark;
+        m_mark = nullptr;
+    }
 }
 
 void DiagnosticItem::setFixitOperations(const ReplacementOperations &replacements)
@@ -505,6 +564,8 @@ QVariant ExplainingStepItem::data(int column, int role) const
             return m_step.message;
         case ClangToolsDiagnosticModel::DiagnosticRole:
             return QVariant::fromValue(static_cast<DiagnosticItem *>(parent())->diagnostic());
+        case ClangToolsDiagnosticModel::DocumentationUrlRole:
+            return parent()->data(column, role);
         case Qt::DisplayRole: {
             const QString mainFilePath = static_cast<DiagnosticItem *>(parent())->diagnostic().location.filePath;
             const QString locationString
@@ -554,39 +615,102 @@ DiagnosticFilterModel::DiagnosticFilterModel(QObject *parent)
                 if (!m_project && project->projectDirectory() == m_lastProjectDirectory)
                     setProject(project);
             });
+    connect(this, &QAbstractItemModel::modelReset, this, [this]() {
+        reset();
+        emit fixitCountersChanged(m_fixitsScheduled, m_fixitsScheduable);
+    });
+    connect(this, &QAbstractItemModel::rowsInserted,
+            this, [this](const QModelIndex &parent, int first, int last) {
+        const Counters counters = countDiagnostics(parent, first, last);
+        m_diagnostics += counters.diagnostics;
+        m_fixitsScheduable += counters.fixits;
+        emit fixitCountersChanged(m_fixitsScheduled, m_fixitsScheduable);
+    });
+    connect(this, &QAbstractItemModel::rowsAboutToBeRemoved,
+            this, [this](const QModelIndex &parent, int first, int last) {
+        const Counters counters = countDiagnostics(parent, first, last);
+        m_diagnostics -= counters.diagnostics;
+        m_fixitsScheduable -= counters.fixits;
+        emit fixitCountersChanged(m_fixitsScheduled, m_fixitsScheduable);
+    });
 }
 
 void DiagnosticFilterModel::setProject(ProjectExplorer::Project *project)
 {
     QTC_ASSERT(project, return);
     if (m_project) {
-        disconnect(ClangToolsProjectSettingsManager::getSettings(m_project),
+        disconnect(ClangToolsProjectSettings::getSettings(m_project).data(),
                    &ClangToolsProjectSettings::suppressedDiagnosticsChanged, this,
                    &DiagnosticFilterModel::handleSuppressedDiagnosticsChanged);
     }
     m_project = project;
     m_lastProjectDirectory = m_project->projectDirectory();
-    connect(ClangToolsProjectSettingsManager::getSettings(m_project),
+    connect(ClangToolsProjectSettings::getSettings(m_project).data(),
             &ClangToolsProjectSettings::suppressedDiagnosticsChanged,
             this, &DiagnosticFilterModel::handleSuppressedDiagnosticsChanged);
     handleSuppressedDiagnosticsChanged();
 }
 
-void DiagnosticFilterModel::addSuppressedDiagnostic(
-        const SuppressedDiagnostic &diag)
+void DiagnosticFilterModel::addSuppressedDiagnostic(const SuppressedDiagnostic &diag)
 {
     QTC_ASSERT(!m_project, return);
     m_suppressedDiagnostics << diag;
     invalidate();
 }
 
-void DiagnosticFilterModel::invalidateFilter()
+void DiagnosticFilterModel::onFixitStatusChanged(const QModelIndex &sourceIndex,
+                                                 FixitStatus oldStatus,
+                                                 FixitStatus newStatus)
 {
-    QSortFilterProxyModel::invalidateFilter();
+    if (!mapFromSource(sourceIndex).isValid())
+        return;
+
+    if (newStatus == FixitStatus::Scheduled)
+        ++m_fixitsScheduled;
+    else if (oldStatus == FixitStatus::Scheduled) {
+        --m_fixitsScheduled;
+        if (newStatus != FixitStatus::NotScheduled)
+            --m_fixitsScheduable;
+    }
+
+    emit fixitCountersChanged(m_fixitsScheduled, m_fixitsScheduable);
 }
 
-bool DiagnosticFilterModel::filterAcceptsRow(int sourceRow,
-        const QModelIndex &sourceParent) const
+void DiagnosticFilterModel::reset()
+{
+    m_filterOptions.reset();
+
+    m_fixitsScheduled = 0;
+    m_fixitsScheduable = 0;
+    m_diagnostics = 0;
+}
+
+DiagnosticFilterModel::Counters DiagnosticFilterModel::countDiagnostics(const QModelIndex &parent,
+                                                                        int first,
+                                                                        int last) const
+{
+    Counters counters;
+    const auto countItem = [&](Utils::TreeItem *item){
+        if (!mapFromSource(item->index()).isValid())
+            return; // Do not count filtered out items.
+        ++counters.diagnostics;
+        if (static_cast<DiagnosticItem *>(item)->diagnostic().hasFixits)
+            ++counters.fixits;
+    };
+
+    auto model = static_cast<ClangToolsDiagnosticModel *>(sourceModel());
+    for (int row = first; row <= last; ++row) {
+        Utils::TreeItem *treeItem = model->itemForIndex(mapToSource(index(row, 0, parent)));
+        if (treeItem->level() == 1)
+            static_cast<FilePathItem *>(treeItem)->forChildrenAtLevel(1, countItem);
+        else if (treeItem->level() == 2)
+            countItem(treeItem);
+    }
+
+    return counters;
+}
+
+bool DiagnosticFilterModel::filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const
 {
     auto model = static_cast<ClangToolsDiagnosticModel *>(sourceModel());
 
@@ -609,9 +733,13 @@ bool DiagnosticFilterModel::filterAcceptsRow(int sourceRow,
     if (parentItem->level() == 1) {
         auto filePathItem = static_cast<FilePathItem *>(parentItem);
         auto diagnosticItem = static_cast<DiagnosticItem *>(filePathItem->childAt(sourceRow));
-
-        // Is the diagnostic explicitly suppressed?
         const Diagnostic &diag = diagnosticItem->diagnostic();
+
+        // Filtered out?
+        if (m_filterOptions && !m_filterOptions->checks.contains(diag.name))
+            return false;
+
+        // Explicitly suppressed?
         foreach (const SuppressedDiagnostic &d, m_suppressedDiagnostics) {
             if (d.description != diag.description)
                 continue;
@@ -623,8 +751,7 @@ bool DiagnosticFilterModel::filterAcceptsRow(int sourceRow,
                 return false;
         }
 
-        // Does the diagnostic match the filter?
-        return diag.description.contains(filterRegExp());
+        return true;
     }
 
     return true; // ExplainingStepItem
@@ -676,9 +803,20 @@ void DiagnosticFilterModel::handleSuppressedDiagnosticsChanged()
 {
     QTC_ASSERT(m_project, return);
     m_suppressedDiagnostics
-            = ClangToolsProjectSettingsManager::getSettings(m_project)->suppressedDiagnostics();
+            = ClangToolsProjectSettings::getSettings(m_project)->suppressedDiagnostics();
     invalidate();
 }
 
+OptionalFilterOptions DiagnosticFilterModel::filterOptions() const
+{
+    return m_filterOptions;
+}
+
+void DiagnosticFilterModel::setFilterOptions(const OptionalFilterOptions &filterOptions)
+{
+    m_filterOptions = filterOptions;
+    invalidateFilter();
+}
+
 } // namespace Internal
 } // namespace ClangTools
