@@ -24,7 +24,6 @@
 ****************************************************************************/
 
 #include "nimbuildconfiguration.h"
-#include "nimbuildsystem.h"
 #include "nimcompilerbuildstep.h"
 #include "nimproject.h"
 
@@ -66,33 +65,27 @@ static FilePath defaultBuildDirectory(const Kit *k,
     return projectDir.pathAppended(buildDirectory);
 }
 
-NimBuildConfiguration::NimBuildConfiguration(Target *target, Core::Id id)
+NimBuildConfiguration::NimBuildConfiguration(Target *target, Utils::Id id)
     : BuildConfiguration(target, id)
 {
     setConfigWidgetDisplayName(tr("General"));
     setConfigWidgetHasFrame(true);
     setBuildDirectorySettingsKey("Nim.NimBuildConfiguration.BuildDirectory");
-}
 
-void NimBuildConfiguration::initialize()
-{
-    BuildConfiguration::initialize();
+    appendInitialBuildStep(Constants::C_NIMCOMPILERBUILDSTEP_ID);
+    appendInitialCleanStep(Constants::C_NIMCOMPILERCLEANSTEP_ID);
 
-    auto bs = qobject_cast<NimBuildSystem *>(project()->buildSystem());
-    QTC_ASSERT(bs, return );
+    setInitializer([this, target](const BuildInfo &info) {
+        // Create the build configuration and initialize it from build info
+        setBuildDirectory(defaultBuildDirectory(target->kit(),
+                                                project()->projectFilePath(),
+                                                displayName(),
+                                                buildType()));
 
-    // Create the build configuration and initialize it from build info
-    setBuildDirectory(defaultBuildDirectory(target()->kit(),
-                                            project()->projectFilePath(),
-                                            displayName(),
-                                            buildType()));
-
-    // Add nim compiler build step
-    {
-        BuildStepList *buildSteps = stepList(ProjectExplorer::Constants::BUILDSTEPS_BUILD);
-        auto nimCompilerBuildStep = new NimCompilerBuildStep(buildSteps);
+        auto nimCompilerBuildStep = buildSteps()->firstOfType<NimCompilerBuildStep>();
+        QTC_ASSERT(nimCompilerBuildStep, return);
         NimCompilerBuildStep::DefaultBuildOptions defaultOption;
-        switch (initialBuildType()) {
+        switch (info.buildType) {
         case BuildConfiguration::Release:
             defaultOption = NimCompilerBuildStep::DefaultBuildOptions::Release;
             break;
@@ -104,19 +97,17 @@ void NimBuildConfiguration::initialize()
             break;
         }
         nimCompilerBuildStep->setDefaultCompilerOptions(defaultOption);
-        Utils::FilePathList nimFiles = bs->nimFiles();
+
+        const Utils::FilePaths nimFiles = project()->files([](const Node *n) {
+            return Project::AllFiles(n) && n->path().endsWith(".nim");
+        });
+
         if (!nimFiles.isEmpty())
             nimCompilerBuildStep->setTargetNimFile(nimFiles.first());
-        buildSteps->appendStep(nimCompilerBuildStep);
-    }
-
-    // Add clean step
-    {
-        BuildStepList *cleanSteps = stepList(ProjectExplorer::Constants::BUILDSTEPS_CLEAN);
-        cleanSteps->appendStep(Constants::C_NIMCOMPILERCLEANSTEP_ID);
-    }
+    });
 }
 
+
 FilePath NimBuildConfiguration::cacheDirectory() const
 {
     return buildDirectory().pathAppended("nimcache");
@@ -131,9 +122,7 @@ FilePath NimBuildConfiguration::outFilePath() const
 
 const NimCompilerBuildStep *NimBuildConfiguration::nimCompilerBuildStep() const
 {
-    BuildStepList *steps = stepList(ProjectExplorer::Constants::BUILDSTEPS_BUILD);
-    QTC_ASSERT(steps, return nullptr);
-    foreach (BuildStep *step, steps->steps())
+    foreach (BuildStep *step, buildSteps()->steps())
         if (step->id() == Constants::C_NIMCOMPILERBUILDSTEP_ID)
             return qobject_cast<NimCompilerBuildStep *>(step);
     return nullptr;
@@ -145,31 +134,23 @@ NimBuildConfigurationFactory::NimBuildConfigurationFactory()
     registerBuildConfiguration<NimBuildConfiguration>(Constants::C_NIMBUILDCONFIGURATION_ID);
     setSupportedProjectType(Constants::C_NIMPROJECT_ID);
     setSupportedProjectMimeTypeName(Constants::C_NIM_PROJECT_MIMETYPE);
-}
 
-QList<BuildInfo> NimBuildConfigurationFactory::availableBuilds
-    (const Kit *k, const FilePath &projectPath, bool forSetup) const
-{
-    QList<BuildInfo> result;
-    for (auto buildType : {BuildConfiguration::Debug, BuildConfiguration::Release}) {
-        BuildInfo info(this);
-        info.buildType = buildType;
-        info.kitId = k->id();
-
-        if (buildType == BuildConfiguration::Debug)
-            info.typeName = tr("Debug");
-        else if (buildType == BuildConfiguration::Profile)
-            info.typeName = tr("Profile");
-        else if (buildType == BuildConfiguration::Release)
-            info.typeName = tr("Release");
-
-        if (forSetup) {
-            info.displayName = info.typeName;
-            info.buildDirectory = defaultBuildDirectory(k, projectPath, info.typeName, buildType);
-        }
-        result.push_back(info);
-    }
-    return result;
+    setBuildGenerator([](const Kit *k, const FilePath &projectPath, bool forSetup) {
+        const auto oneBuild = [&](BuildConfiguration::BuildType buildType, const QString &typeName) {
+            BuildInfo info;
+            info.buildType = buildType;
+            info.typeName = typeName;
+            if (forSetup) {
+                info.displayName = info.typeName;
+                info.buildDirectory = defaultBuildDirectory(k, projectPath, info.typeName, buildType);
+            }
+            return info;
+        };
+        return QList<BuildInfo>{
+            oneBuild(BuildConfiguration::Debug, BuildConfiguration::tr("Debug")),
+            oneBuild(BuildConfiguration::Release, BuildConfiguration::tr("Release"))
+        };
+    });
 }
 
 } // namespace Nim
