@@ -29,36 +29,38 @@
 #include <coreplugin/fileiconprovider.h>
 #include <coreplugin/icore.h>
 
-#include <utils/runextensions.h>
 #include <utils/algorithm.h>
+#include <utils/fancylineedit.h>
+#include <utils/pathchooser.h>
+#include <utils/runextensions.h>
+#include <utils/stringutils.h>
 
 #include <QDialogButtonBox>
+#include <QDir>
 #include <QGridLayout>
 #include <QHBoxLayout>
 #include <QLineEdit>
 #include <QPushButton>
 #include <QTreeView>
-#include <QDir>
-#include <utils/pathchooser.h>
 
 namespace ProjectExplorer {
 
 const char HIDE_FILE_FILTER_DEFAULT[] = "Makefile*; *.o; *.lo; *.la; *.obj; *~; *.files;"
                                         " *.config; *.creator; *.user*; *.includes; *.autosave";
-const char SHOW_FILE_FILTER_DEFAULT[] = "*.c; *.cc; *.cpp; *.cp; *.cxx; *.c++; *.h; *.hh; *.hpp; *.hxx;";
+const char SELECT_FILE_FILTER_DEFAULT[] = "*.c; *.cc; *.cpp; *.cp; *.cxx; *.c++; *.h; *.hh; *.hpp; *.hxx;";
 
 SelectableFilesModel::SelectableFilesModel(QObject *parent) : QAbstractItemModel(parent)
 {
     m_root = new Tree;
 }
 
-void SelectableFilesModel::setInitialMarkedFiles(const Utils::FileNameList &files)
+void SelectableFilesModel::setInitialMarkedFiles(const Utils::FilePaths &files)
 {
-    m_files = files.toSet();
+    m_files = Utils::toSet(files);
     m_allFiles = files.isEmpty();
 }
 
-void SelectableFilesFromDirModel::startParsing(const Utils::FileName &baseDir)
+void SelectableFilesFromDirModel::startParsing(const Utils::FilePath &baseDir)
 {
     m_watcher.cancel();
     m_watcher.waitForFinished();
@@ -86,7 +88,7 @@ void SelectableFilesFromDirModel::buildTreeFinished()
     m_root = m_rootForFuture;
     m_rootForFuture = nullptr;
     m_outOfBaseDirFiles
-            = Utils::filtered(m_files, [this](const Utils::FileName &fn) { return !fn.isChildOf(m_baseDir); });
+            = Utils::filtered(m_files, [this](const Utils::FilePath &fn) { return !fn.isChildOf(m_baseDir); });
 
     endResetModel();
     emit parsingFinished();
@@ -109,14 +111,13 @@ SelectableFilesModel::FilterState SelectableFilesModel::filter(Tree *t)
         return g.isMatch(t->name);
     };
 
-    //If one of the "show file" filters matches just return false
-    if (Utils::anyOf(m_showFilesFilter, matchesTreeName))
+    if (Utils::anyOf(m_selectFilesFilter, matchesTreeName))
         return FilterState::CHECKED;
 
     return Utils::anyOf(m_hideFilesFilter, matchesTreeName) ? FilterState::HIDDEN : FilterState::SHOWN;
 }
 
-void SelectableFilesFromDirModel::buildTree(const Utils::FileName &baseDir, Tree *tree,
+void SelectableFilesFromDirModel::buildTree(const Utils::FilePath &baseDir, Tree *tree,
                                             QFutureInterface<void> &fi, int symlinkDepth)
 {
     if (symlinkDepth == 0)
@@ -127,8 +128,8 @@ void SelectableFilesFromDirModel::buildTree(const Utils::FileName &baseDir, Tree
                                                                               QDir::NoDotAndDotDot);
     bool allChecked = true;
     bool allUnchecked = true;
-    foreach (const QFileInfo &fileInfo, fileInfoList) {
-        Utils::FileName fn = Utils::FileName(fileInfo);
+    for (const QFileInfo &fileInfo : fileInfoList) {
+        Utils::FilePath fn = Utils::FilePath::fromFileInfo(fileInfo);
         if (m_futureCount % 100) {
             emit parsingProgress(fn);
             if (fi.isCanceled())
@@ -298,36 +299,36 @@ void SelectableFilesModel::propagateDown(const QModelIndex &idx)
 
 Qt::ItemFlags SelectableFilesModel::flags(const QModelIndex &index) const
 {
-    Q_UNUSED(index);
+    Q_UNUSED(index)
     return Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsUserCheckable;
 }
 
-Utils::FileNameList SelectableFilesModel::selectedPaths() const
+Utils::FilePaths SelectableFilesModel::selectedPaths() const
 {
-    Utils::FileNameList result;
+    Utils::FilePaths result;
     collectPaths(m_root, &result);
     return result;
 }
 
-void SelectableFilesModel::collectPaths(Tree *root, Utils::FileNameList *result)  const
+void SelectableFilesModel::collectPaths(Tree *root, Utils::FilePaths *result)  const
 {
     if (root->checked == Qt::Unchecked)
         return;
     result->append(root->fullPath);
-    foreach (Tree *t, root->childDirectories)
+    for (Tree *t : qAsConst(root->childDirectories))
         collectPaths(t, result);
 }
 
-Utils::FileNameList SelectableFilesModel::selectedFiles() const
+Utils::FilePaths SelectableFilesModel::selectedFiles() const
 {
-    Utils::FileNameList result = m_outOfBaseDirFiles.toList();
+    Utils::FilePaths result = Utils::toList(m_outOfBaseDirFiles);
     collectFiles(m_root, &result);
     return result;
 }
 
-Utils::FileNameList SelectableFilesModel::preservedFiles() const
+Utils::FilePaths SelectableFilesModel::preservedFiles() const
 {
-    return m_outOfBaseDirFiles.toList();
+    return Utils::toList(m_outOfBaseDirFiles);
 }
 
 bool SelectableFilesModel::hasCheckedFiles() const
@@ -335,13 +336,13 @@ bool SelectableFilesModel::hasCheckedFiles() const
     return m_root->checked != Qt::Unchecked;
 }
 
-void SelectableFilesModel::collectFiles(Tree *root, Utils::FileNameList *result) const
+void SelectableFilesModel::collectFiles(Tree *root, Utils::FilePaths *result) const
 {
     if (root->checked == Qt::Unchecked)
         return;
-    foreach (Tree *t, root->childDirectories)
+    for (Tree *t : qAsConst(root->childDirectories))
         collectFiles(t, result);
-    foreach (Tree *t, root->visibleFiles)
+    for (Tree *t : qAsConst(root->visibleFiles))
         if (t->checked == Qt::Checked)
             result->append(t->fullPath);
 }
@@ -349,8 +350,8 @@ void SelectableFilesModel::collectFiles(Tree *root, Utils::FileNameList *result)
 QList<Glob> SelectableFilesModel::parseFilter(const QString &filter)
 {
     QList<Glob> result;
-    QStringList list = filter.split(QLatin1Char(';'), QString::SkipEmptyParts);
-    foreach (const QString &e, list) {
+    const QStringList list = filter.split(QLatin1Char(';'), Utils::SkipEmptyParts);
+    for (const QString &e : list) {
         QString entry = e.trimmed();
         Glob g;
         if (entry.indexOf(QLatin1Char('*')) == -1 && entry.indexOf(QLatin1Char('?')) == -1) {
@@ -362,18 +363,19 @@ QList<Glob> SelectableFilesModel::parseFilter(const QString &filter)
             g.matchString = entry.mid(1);
         } else {
             g.mode = Glob::REGEXP;
-            g.matchRegexp = QRegExp(entry, Qt::CaseInsensitive, QRegExp::Wildcard);
+            const QString re = QRegularExpression::wildcardToRegularExpression(entry);
+            g.matchRegexp = QRegularExpression(re, QRegularExpression::CaseInsensitiveOption);
         }
         result.append(g);
     }
     return result;
 }
 
-void SelectableFilesModel::applyFilter(const QString &showFilesfilter, const QString &hideFilesfilter)
+void SelectableFilesModel::applyFilter(const QString &selectFilesfilter, const QString &hideFilesfilter)
 {
-    QList<Glob> filter = parseFilter(showFilesfilter);
-    bool mustApply = filter != m_showFilesFilter;
-    m_showFilesFilter = filter;
+    QList<Glob> filter = parseFilter(selectFilesfilter);
+    bool mustApply = filter != m_selectFilesFilter;
+    m_selectFilesFilter = filter;
 
     filter = parseFilter(hideFilesfilter);
     mustApply = mustApply || (filter != m_hideFilesFilter);
@@ -392,10 +394,10 @@ void SelectableFilesModel::selectAllFiles(Tree *root)
 {
     root->checked = Qt::Checked;
 
-    foreach (Tree *t, root->childDirectories)
+    for (Tree *t : qAsConst(root->childDirectories))
         selectAllFiles(t);
 
-    foreach (Tree *t, root->visibleFiles)
+    for (Tree *t : qAsConst(root->visibleFiles))
         t->checked = Qt::Checked;
 
     emit checkedFilesChanged();
@@ -493,7 +495,9 @@ Qt::CheckState SelectableFilesModel::applyFilter(const QModelIndex &idx)
     }
 
     for (int i=0; i < t->visibleFiles.size(); ++i) {
-        if (t->visibleFiles.at(i)->checked == Qt::Checked)
+        Tree * const fileNode = t->visibleFiles.at(i);
+        fileNode->checked = filter(fileNode) == FilterState::CHECKED ? Qt::Checked : Qt::Unchecked;
+        if (fileNode->checked)
             allUnchecked = false;
         else
             allChecked = false;
@@ -521,7 +525,7 @@ Qt::CheckState SelectableFilesModel::applyFilter(const QModelIndex &idx)
 namespace {
 
 enum class SelectableFilesWidgetRows {
-    BaseDirectory, ShowFileFilter, HideFileFilter, ApplyButton, View, Progress, PreservedInformation
+    BaseDirectory, SelectFileFilter, HideFileFilter, ApplyButton, View, Progress, PreservedInformation
 };
 
 } // namespace
@@ -531,24 +535,24 @@ SelectableFilesWidget::SelectableFilesWidget(QWidget *parent) :
     m_baseDirChooser(new Utils::PathChooser),
     m_baseDirLabel(new QLabel),
     m_startParsingButton(new QPushButton),
-    m_showFilesFilterLabel(new QLabel),
-    m_showFilesFilterEdit(new QLineEdit),
+    m_selectFilesFilterLabel(new QLabel),
+    m_selectFilesFilterEdit(new Utils::FancyLineEdit),
     m_hideFilesFilterLabel(new QLabel),
-    m_hideFilesFilterEdit(new QLineEdit),
-    m_applyFilterButton(new QPushButton),
+    m_hideFilesFilterEdit(new Utils::FancyLineEdit),
+    m_applyFiltersButton(new QPushButton),
     m_view(new QTreeView),
     m_preservedFilesLabel(new QLabel),
     m_progressLabel(new QLabel)
 {
-    const QString showFilter
+    const QString selectFilter
             = Core::ICore::settings()->value("GenericProject/ShowFileFilter",
-                                             QLatin1String(SHOW_FILE_FILTER_DEFAULT)).toString();
+                                             QLatin1String(SELECT_FILE_FILTER_DEFAULT)).toString();
     const QString hideFilter
             = Core::ICore::settings()->value("GenericProject/FileFilter",
                                              QLatin1String(HIDE_FILE_FILTER_DEFAULT)).toString();
 
     auto layout = new QGridLayout(this);
-    layout->setMargin(0);
+    layout->setContentsMargins(0, 0, 0, 0);
 
     m_baseDirLabel->setText(tr("Source directory:"));
     m_baseDirChooser->setHistoryCompleter(QLatin1String("PE.AddToProjectDir.History"));
@@ -561,22 +565,22 @@ SelectableFilesWidget::SelectableFilesWidget(QWidget *parent) :
     connect(m_baseDirChooser, &Utils::PathChooser::validChanged,
             this, &SelectableFilesWidget::baseDirectoryChanged);
     connect(m_startParsingButton, &QAbstractButton::clicked,
-            this, [this]() { startParsing(m_baseDirChooser->fileName()); });
+            this, [this]() { startParsing(m_baseDirChooser->filePath()); });
 
-    m_showFilesFilterLabel->setText(tr("Show files matching:"));
-    m_showFilesFilterEdit->setText(showFilter);
-    layout->addWidget(m_showFilesFilterLabel, static_cast<int>(SelectableFilesWidgetRows::ShowFileFilter), 0);
-    layout->addWidget(m_showFilesFilterEdit, static_cast<int>(SelectableFilesWidgetRows::ShowFileFilter), 1, 1, 3);
+    m_selectFilesFilterLabel->setText(tr("Select files matching:"));
+    m_selectFilesFilterEdit->setText(selectFilter);
+    layout->addWidget(m_selectFilesFilterLabel, static_cast<int>(SelectableFilesWidgetRows::SelectFileFilter), 0);
+    layout->addWidget(m_selectFilesFilterEdit, static_cast<int>(SelectableFilesWidgetRows::SelectFileFilter), 1, 1, 3);
 
     m_hideFilesFilterLabel->setText(tr("Hide files matching:"));
     m_hideFilesFilterEdit->setText(hideFilter);
     layout->addWidget(m_hideFilesFilterLabel, static_cast<int>(SelectableFilesWidgetRows::HideFileFilter), 0);
     layout->addWidget(m_hideFilesFilterEdit, static_cast<int>(SelectableFilesWidgetRows::HideFileFilter), 1, 1, 3);
 
-    m_applyFilterButton->setText(tr("Apply Filter"));
-    layout->addWidget(m_applyFilterButton, static_cast<int>(SelectableFilesWidgetRows::ApplyButton), 3);
+    m_applyFiltersButton->setText(tr("Apply Filters"));
+    layout->addWidget(m_applyFiltersButton, static_cast<int>(SelectableFilesWidgetRows::ApplyButton), 3);
 
-    connect(m_applyFilterButton, &QAbstractButton::clicked,
+    connect(m_applyFiltersButton, &QAbstractButton::clicked,
             this, &SelectableFilesWidget::applyFilter);
 
     m_view->setMinimumSize(500, 400);
@@ -589,8 +593,8 @@ SelectableFilesWidget::SelectableFilesWidget(QWidget *parent) :
     layout->addWidget(m_progressLabel, static_cast<int>(SelectableFilesWidgetRows::Progress), 0, 1, 4);
 }
 
-SelectableFilesWidget::SelectableFilesWidget(const Utils::FileName &path,
-                                             const Utils::FileNameList &files, QWidget *parent) :
+SelectableFilesWidget::SelectableFilesWidget(const Utils::FilePath &path,
+                                             const Utils::FilePaths &files, QWidget *parent) :
     SelectableFilesWidget(parent)
 {
     resetModel(path, files);
@@ -598,7 +602,11 @@ SelectableFilesWidget::SelectableFilesWidget(const Utils::FileName &path,
 
 void SelectableFilesWidget::setAddFileFilter(const QString &filter)
 {
-    m_showFilesFilterEdit->setText(filter);
+    m_selectFilesFilterEdit->setText(filter);
+    if (m_applyFiltersButton->isEnabled())
+        applyFilter();
+    else
+        m_filteringScheduled = true;
 }
 
 void SelectableFilesWidget::setBaseDirEditable(bool edit)
@@ -609,14 +617,14 @@ void SelectableFilesWidget::setBaseDirEditable(bool edit)
     m_startParsingButton->setVisible(edit);
 }
 
-Utils::FileNameList SelectableFilesWidget::selectedFiles() const
+Utils::FilePaths SelectableFilesWidget::selectedFiles() const
 {
-    return m_model ? m_model->selectedFiles() : Utils::FileNameList();
+    return m_model ? m_model->selectedFiles() : Utils::FilePaths();
 }
 
-Utils::FileNameList SelectableFilesWidget::selectedPaths() const
+Utils::FilePaths SelectableFilesWidget::selectedPaths() const
 {
-    return m_model ? m_model->selectedPaths() : Utils::FileNameList();
+    return m_model ? m_model->selectedPaths() : Utils::FilePaths();
 }
 
 bool SelectableFilesWidget::hasFilesSelected() const
@@ -624,7 +632,7 @@ bool SelectableFilesWidget::hasFilesSelected() const
     return m_model ? m_model->hasCheckedFiles() : false;
 }
 
-void SelectableFilesWidget::resetModel(const Utils::FileName &path, const Utils::FileNameList &files)
+void SelectableFilesWidget::resetModel(const Utils::FilePath &path, const Utils::FilePaths &files)
 {
     m_view->setModel(nullptr);
 
@@ -639,7 +647,7 @@ void SelectableFilesWidget::resetModel(const Utils::FileName &path, const Utils:
     connect(m_model, &SelectableFilesModel::checkedFilesChanged,
             this, &SelectableFilesWidget::selectedFilesChanged);
 
-    m_baseDirChooser->setFileName(path);
+    m_baseDirChooser->setFilePath(path);
     m_view->setModel(m_model);
 
     startParsing(path);
@@ -651,11 +659,17 @@ void SelectableFilesWidget::cancelParsing()
         m_model->cancel();
 }
 
+void SelectableFilesWidget::enableFilterHistoryCompletion(const QString &keyPrefix)
+{
+    m_selectFilesFilterEdit->setHistoryCompleter(keyPrefix + ".select", true);
+    m_hideFilesFilterEdit->setHistoryCompleter(keyPrefix + ".hide", true);
+}
+
 void SelectableFilesWidget::enableWidgets(bool enabled)
 {
     m_hideFilesFilterEdit->setEnabled(enabled);
-    m_showFilesFilterEdit->setEnabled(enabled);
-    m_applyFilterButton->setEnabled(enabled);
+    m_selectFilesFilterEdit->setEnabled(enabled);
+    m_applyFiltersButton->setEnabled(enabled);
     m_view->setEnabled(enabled);
     m_baseDirChooser->setEnabled(enabled);
 
@@ -667,8 +681,9 @@ void SelectableFilesWidget::enableWidgets(bool enabled)
 
 void SelectableFilesWidget::applyFilter()
 {
+    m_filteringScheduled = false;
     if (m_model)
-        m_model->applyFilter(m_showFilesFilterEdit->text(), m_hideFilesFilterEdit->text());
+        m_model->applyFilter(m_selectFilesFilterEdit->text(), m_hideFilesFilterEdit->text());
 }
 
 void SelectableFilesWidget::baseDirectoryChanged(bool validState)
@@ -676,7 +691,7 @@ void SelectableFilesWidget::baseDirectoryChanged(bool validState)
     m_startParsingButton->setEnabled(validState);
 }
 
-void SelectableFilesWidget::startParsing(const Utils::FileName &baseDir)
+void SelectableFilesWidget::startParsing(const Utils::FilePath &baseDir)
 {
     if (!m_model)
         return;
@@ -686,7 +701,7 @@ void SelectableFilesWidget::startParsing(const Utils::FileName &baseDir)
     m_model->startParsing(baseDir);
 }
 
-void SelectableFilesWidget::parsingProgress(const Utils::FileName &fileName)
+void SelectableFilesWidget::parsingProgress(const Utils::FilePath &fileName)
 {
     m_progressLabel->setText(tr("Generating file list...\n\n%1").arg(fileName.toUserOutput()));
 }
@@ -698,11 +713,13 @@ void SelectableFilesWidget::parsingFinished()
 
     smartExpand(m_model->index(0,0, QModelIndex()));
 
-    const Utils::FileNameList preservedFiles = m_model->preservedFiles();
+    const Utils::FilePaths preservedFiles = m_model->preservedFiles();
     m_preservedFilesLabel->setText(tr("Not showing %n files that are outside of the base directory.\n"
                                       "These files are preserved.", nullptr, preservedFiles.count()));
 
     enableWidgets(true);
+    if (m_filteringScheduled)
+        applyFilter();
 }
 
 void SelectableFilesWidget::smartExpand(const QModelIndex &idx)
@@ -720,8 +737,8 @@ void SelectableFilesWidget::smartExpand(const QModelIndex &idx)
 // SelectableFilesDialogs
 //////////
 
-SelectableFilesDialogEditFiles::SelectableFilesDialogEditFiles(const Utils::FileName &path,
-                                                               const Utils::FileNameList &files,
+SelectableFilesDialogEditFiles::SelectableFilesDialogEditFiles(const Utils::FilePath &path,
+                                                               const Utils::FilePaths &files,
                                                                QWidget *parent) :
     QDialog(parent),
     m_filesWidget(new SelectableFilesWidget(path, files))
@@ -732,6 +749,7 @@ SelectableFilesDialogEditFiles::SelectableFilesDialogEditFiles(const Utils::File
     layout->addWidget(m_filesWidget);
 
     m_filesWidget->setBaseDirEditable(false);
+    m_filesWidget->enableFilterHistoryCompletion(Constants::ADD_FILES_DIALOG_FILTER_HISTORY_KEY);
 
     auto buttonBox = new QDialogButtonBox(Qt::Horizontal, this);
     buttonBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
@@ -743,18 +761,19 @@ SelectableFilesDialogEditFiles::SelectableFilesDialogEditFiles(const Utils::File
     layout->addWidget(buttonBox);
 }
 
-Utils::FileNameList SelectableFilesDialogEditFiles::selectedFiles() const
+Utils::FilePaths SelectableFilesDialogEditFiles::selectedFiles() const
 {
     return m_filesWidget->selectedFiles();
 }
 
+
 //////////
 // SelectableFilesDialogAddDirectory
 //////////
 
 
-SelectableFilesDialogAddDirectory::SelectableFilesDialogAddDirectory(const Utils::FileName &path,
-                                                                     const Utils::FileNameList &files,
+SelectableFilesDialogAddDirectory::SelectableFilesDialogAddDirectory(const Utils::FilePath &path,
+                                                                     const Utils::FilePaths &files,
                                                                      QWidget *parent) :
     SelectableFilesDialogEditFiles(path, files, parent)
 {
