@@ -33,7 +33,11 @@
 #include <nodehints.h>
 #include <nodemetainfo.h>
 
+#include <designdocument.h>
+#include <qmldesignerplugin.h>
+
 #include <utils/algorithm.h>
+#include <utils/qtcassert.h>
 
 #include <QVariant>
 #include <QMetaProperty>
@@ -64,6 +68,13 @@ void ItemLibraryModel::setExpanded(bool expanded, const QString &section)
         collapsedStateHash.insert(section, expanded);
 }
 
+void ItemLibraryModel::setFlowMode(bool b)
+{
+    m_flowMode = b;
+    bool changed;
+    updateVisibility(&changed);
+}
+
 ItemLibraryModel::ItemLibraryModel(QObject *parent)
     : QAbstractListModel(parent)
 {
@@ -179,18 +190,68 @@ void ItemLibraryModel::update(ItemLibraryInfo *itemLibraryInfo, Model *model)
 
         qCInfo(itemlibraryPopulate) << "required import: " << entry.requiredImport() << entryToImport(entry).toImportString();
 
-        if (!isItem && valid) {
-            qDebug() << Q_FUNC_INFO;
-            qDebug() << metaInfo.typeName() << "is not a QtQuick.Item";
-            qDebug() << Utils::transform(metaInfo.superClasses(), &NodeMetaInfo::typeName);
-        }
-
         bool forceVisiblity = valid && NodeHints::fromItemLibraryEntry(entry).visibleInLibrary();
 
-        if (valid
-                && (isItem || forceVisiblity) //We can change if the navigator does support pure QObjects
-                && (entry.requiredImport().isEmpty()
-                    || model->hasImport(entryToImport(entry), true, true))) {
+        if (m_flowMode && metaInfo.isValid()) {
+
+            isItem = metaInfo.isSubclassOf("FlowView.FlowItem")
+                    || metaInfo.isSubclassOf("FlowView.FlowWildcard")
+                    || metaInfo.isSubclassOf("FlowView.FlowDecision");
+            forceVisiblity = isItem;
+        }
+
+        DesignDocument *designDocument = QmlDesignerPlugin::instance()
+                                             ->documentManager()
+                                             .currentDesignDocument();
+
+        if (designDocument && designDocument->isQtForMCUsProject()) {
+            const QList<TypeName> blockTypes = {"QtQuick.AnimatedImage",
+                                                "QtQuick.BorderImage",
+                                                "QtQuick.FocusScope",
+                                                "QtQuick.TextInput",
+                                                "QtQuick.TextEdit",
+                                                "QtQuick.Flow",
+                                                "QtQuick.Grid",
+                                                "QtQuick.GridView",
+                                                "QtQuick.PathView",
+                                                "QtQuick.Controls",
+                                                "QtQuick.Controls.BusyIndicator",
+                                                "QtQuick.Controls.ButtonGroup",
+                                                "QtQuick.Controls.CheckDelegate",
+                                                "QtQuick.Controls.Container",
+                                                "QtQuick.Controls.ComboBox",
+                                                "QtQuick.Controls.DelayButton",
+                                                "QtQuick.Controls.Frame",
+                                                "QtQuick.Controls.GroupBox",
+                                                "QtQuick.Controls.ItemDelegate",
+                                                "QtQuick.Controls.Label",
+                                                "QtQuick.Controls.Page",
+                                                "QtQuick.Controls.PageIndicator",
+                                                "QtQuick.Controls.Pane",
+                                                "QtQuick.Controls.RadioDelegate",
+                                                "QtQuick.Controls.RangeSlider",
+                                                "QtQuick.Controls.RoundButton",
+                                                "QtQuick.Controls.ScrollView",
+                                                "QtQuick.Controls.SpinBox",
+                                                "QtQuick.Controls.StackView",
+                                                "QtQuick.Controls.SwipeDelegate",
+                                                "QtQuick.Controls.SwitchDelegate",
+                                                "QtQuick.Controls.ToolBar",
+                                                "QtQuick.Controls.ToolButton",
+                                                "QtQuick.Controls.TabBar",
+                                                "QtQuick.Controls.TabButton",
+                                                "QtQuick.Controls.TextArea",
+                                                "QtQuick.Controls.TextField",
+                                                "QtQuick.Controls.ToolSeparator",
+                                                "QtQuick.Controls.Tumbler"};
+
+            if (blockTypes.contains(entry.typeName()))
+                valid = false;
+        }
+
+        if (valid && (isItem || forceVisiblity) //We can change if the navigator does support pure QObjects
+            && (entry.requiredImport().isEmpty()
+                || model->hasImport(entryToImport(entry), true, true))) {
             QString itemSectionName = entry.category();
             qCInfo(itemlibraryPopulate) << "Adding:" << entry.typeName() << "to:" << entry.category();
             ItemLibrarySection *sectionModel = sectionByName(itemSectionName);
@@ -227,11 +288,6 @@ QMimeData *ItemLibraryModel::getMimeData(const ItemLibraryEntry &itemLibraryEntr
     return mimeData;
 }
 
-QList<ItemLibrarySection *> ItemLibraryModel::sections() const
-{
-    return m_sections;
-}
-
 void ItemLibraryModel::clearSections()
 {
     qDeleteAll(m_sections);
@@ -262,6 +318,7 @@ void ItemLibraryModel::updateVisibility(bool *changed)
         bool sectionChanged = false;
         bool sectionVisibility = itemLibrarySection->updateSectionVisibility(sectionSearchText,
                                                                              &sectionChanged);
+
         *changed |= sectionChanged;
         *changed |= itemLibrarySection->setVisible(sectionVisibility);
     }
@@ -279,13 +336,15 @@ void ItemLibraryModel::addRoleNames()
 
 void ItemLibraryModel::sortSections()
 {
+    int nullPointerSectionCount = m_sections.removeAll(QPointer<ItemLibrarySection>());
+    QTC_ASSERT(nullPointerSectionCount == 0,;);
     auto sectionSort = [](ItemLibrarySection *first, ItemLibrarySection *second) {
-        return QString::localeAwareCompare(first->sortingName(), second->sortingName()) < 1;
+        return QString::localeAwareCompare(first->sortingName(), second->sortingName()) < 0;
     };
 
     std::sort(m_sections.begin(), m_sections.end(), sectionSort);
 
-    foreach (ItemLibrarySection *itemLibrarySection, m_sections)
+    for (const auto &itemLibrarySection : qAsConst(m_sections))
         itemLibrarySection->sortItems();
 }
 
