@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,26 +9,22 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "qmljscompletionassist.h"
+#include "qmljseditor.h"
 #include "qmljseditorconstants.h"
 #include "qmljsreuse.h"
 #include "qmlexpressionundercursor.h"
@@ -83,7 +79,7 @@ enum CompletionOrder {
     TypeOrder = -30
 };
 
-static void addCompletion(QList<AssistProposalItem *> *completions,
+static void addCompletion(QList<AssistProposalItemInterface *> *completions,
                           const QString &text,
                           const QIcon &icon,
                           int order,
@@ -100,7 +96,7 @@ static void addCompletion(QList<AssistProposalItem *> *completions,
     completions->append(item);
 }
 
-static void addCompletions(QList<AssistProposalItem *> *completions,
+static void addCompletions(QList<AssistProposalItemInterface *> *completions,
                            const QStringList &newCompletions,
                            const QIcon &icon,
                            int order)
@@ -125,10 +121,10 @@ public:
 class CompletionAdder : public PropertyProcessor
 {
 protected:
-    QList<AssistProposalItem *> *completions;
+    QList<AssistProposalItemInterface *> *completions;
 
 public:
-    CompletionAdder(QList<AssistProposalItem *> *completions,
+    CompletionAdder(QList<AssistProposalItemInterface *> *completions,
                     const QIcon &icon, int order)
         : completions(completions)
         , icon(icon)
@@ -142,7 +138,7 @@ public:
         if (const FunctionValue *func = value->asFunctionValue()) {
             // constructors usually also have other interesting members,
             // don't consider them pure functions and complete the '()'
-            if (!func->lookupMember(QLatin1String("prototype"), 0, 0, false))
+            if (!func->lookupMember(QLatin1String("prototype"), nullptr, nullptr, false))
                 data = QVariant::fromValue(CompleteFunctionCall(func->namedArgumentCount() || func->isVariadic()));
         }
         addCompletion(completions, name, icon, order, data);
@@ -155,7 +151,7 @@ public:
 class LhsCompletionAdder : public CompletionAdder
 {
 public:
-    LhsCompletionAdder(QList<AssistProposalItem *> *completions,
+    LhsCompletionAdder(QList<AssistProposalItemInterface *> *completions,
                        const QIcon &icon,
                        int order,
                        bool afterOn)
@@ -189,21 +185,16 @@ public:
 class ProcessProperties: private MemberProcessor
 {
     QSet<const ObjectValue *> _processed;
-    bool _globalCompletion;
-    bool _enumerateGeneratedSlots;
-    bool _enumerateSlots;
+    bool _globalCompletion = false;
+    bool _enumerateGeneratedSlots = false;
+    bool _enumerateMethods = true;
     const ScopeChain *_scopeChain;
-    const ObjectValue *_currentObject;
-    PropertyProcessor *_propertyProcessor;
+    const ObjectValue *_currentObject = nullptr;
+    PropertyProcessor *_propertyProcessor = nullptr;
 
 public:
     ProcessProperties(const ScopeChain *scopeChain)
-        : _globalCompletion(false),
-          _enumerateGeneratedSlots(false),
-          _enumerateSlots(true),
-          _scopeChain(scopeChain),
-          _currentObject(0),
-          _propertyProcessor(0)
+        : _scopeChain(scopeChain)
     {
     }
 
@@ -217,9 +208,9 @@ public:
         _enumerateGeneratedSlots = enumerate;
     }
 
-    void setEnumerateSlots(bool enumerate)
+    void setEnumerateMethods(bool enumerate)
     {
-        _enumerateSlots = enumerate;
+        _enumerateMethods = enumerate;
     }
 
     void operator ()(const Value *value, PropertyProcessor *processor)
@@ -260,14 +251,14 @@ private:
 
     bool processSignal(const QString &name, const Value *value) override
     {
-        if (_globalCompletion)
+        if (_globalCompletion || _enumerateMethods)
             process(name, value);
         return true;
     }
 
     bool processSlot(const QString &name, const Value *value) override
     {
-        if (_enumerateSlots)
+        if (_enumerateMethods)
             process(name, value);
         return true;
     }
@@ -299,7 +290,7 @@ private:
 
         _currentObject = object;
         object->processMembers(this);
-        _currentObject = 0;
+        _currentObject = nullptr;
     }
 };
 
@@ -308,16 +299,16 @@ const Value *getPropertyValue(const ObjectValue *object,
                                            const ContextPtr &context)
 {
     if (propertyNames.isEmpty() || !object)
-        return 0;
+        return nullptr;
 
     const Value *value = object;
     foreach (const QString &name, propertyNames) {
         if (const ObjectValue *objectValue = value->asObjectValue()) {
             value = objectValue->lookupMember(name, context);
             if (!value)
-                return 0;
+                return nullptr;
         } else {
-            return 0;
+            return nullptr;
         }
     }
     return value;
@@ -335,6 +326,42 @@ bool isLiteral(AST::Node *ast)
 
 } // Anonymous
 
+QStringList qmlJSAutoComplete(QTextDocument *textDocument,
+                              int position,
+                              const QString &fileName,
+                              TextEditor::AssistReason reason,
+                              const SemanticInfo &info)
+{
+    QStringList list;
+    QmlJSCompletionAssistProcessor processor;
+    QScopedPointer<IAssistProposal> proposal(processor.perform( /* The processor takes ownership. */
+                                                 new QmlJSCompletionAssistInterface(
+                                                     textDocument,
+                                                     position,
+                                                     fileName,
+                                                     reason,
+                                                     info)));
+
+    if (proposal) {
+        GenericProposalModelPtr model = proposal->model().staticCast<GenericProposalModel>();
+
+        int basePosition = proposal->basePosition();
+        const QString prefix = textDocument->toPlainText().mid(basePosition,
+                                                               position - basePosition);
+
+        if (reason == TextEditor::ExplicitlyInvoked) {
+            model->filter(prefix);
+            model->sort(prefix);
+        }
+
+        for (int i = 0; i < model->size(); ++i)
+            list.append(proposal->model()->text(i).trimmed());
+        list.append(prefix);
+    }
+
+    return list;
+}
+
 } // namesapce QmlJSEditor
 
 Q_DECLARE_METATYPE(QmlJSEditor::CompleteFunctionCall)
@@ -353,12 +380,11 @@ bool QmlJSAssistProposalItem::prematurelyApplies(const QChar &c) const
             || (text().endsWith(QLatin1Char('.')) && c == QLatin1Char('.'));
 }
 
-void QmlJSAssistProposalItem::applyContextualContent(TextEditorWidget *editorWidget,
-                                                      int basePosition) const
+void QmlJSAssistProposalItem::applyContextualContent(TextEditor::TextDocumentManipulatorInterface &manipulator,
+                                                     int basePosition) const
 {
-    const int currentPosition = editorWidget->position();
-    editorWidget->setCursorPosition(basePosition);
-    editorWidget->remove(currentPosition - basePosition);
+    const int currentPosition = manipulator.currentPosition();
+    manipulator.replace(basePosition, currentPosition - basePosition, QString());
 
     QString content = text();
     int cursorOffset = 0;
@@ -377,16 +403,18 @@ void QmlJSAssistProposalItem::applyContextualContent(TextEditorWidget *editorWid
     int replacedLength = 0;
     for (int i = 0; i < replaceable.length(); ++i) {
         const QChar a = replaceable.at(i);
-        const QChar b = editorWidget->characterAt(editorWidget->position() + i);
+        const QChar b = manipulator.characterAt(manipulator.currentPosition() + i);
         if (a == b)
             ++replacedLength;
         else
             break;
     }
-    const int length = editorWidget->position() - basePosition + replacedLength;
-    editorWidget->replace(length, content);
-    if (cursorOffset)
-        editorWidget->setCursorPosition(editorWidget->position() + cursorOffset);
+    const int length = manipulator.currentPosition() - basePosition + replacedLength;
+    manipulator.replace(basePosition, length, content);
+    if (cursorOffset) {
+        manipulator.setCursorPosition(manipulator.currentPosition() + cursorOffset);
+        manipulator.setAutoCompleteSkipPosition(manipulator.currentPosition());
+    }
 }
 
 // -------------------------
@@ -438,7 +466,7 @@ QString FunctionHintProposalModel::text(int index) const
         prettyMethod += arg;
     }
     if (m_isVariadic) {
-        if (m_namedArguments.size())
+        if (!m_namedArguments.isEmpty())
             prettyMethod += QLatin1String(", ");
         prettyMethod += QLatin1String("...");
     }
@@ -454,8 +482,7 @@ int FunctionHintProposalModel::activeArgument(const QString &prefix) const
     int parcount = 0;
     Scanner tokenize;
     const QList<Token> tokens = tokenize(prefix);
-    for (int i = 0; i < tokens.count(); ++i) {
-        const Token &tk = tokens.at(i);
+    for (auto &tk : tokens) {
         if (tk.is(Token::LeftParenthesis))
             ++parcount;
         else if (tk.is(Token::RightParenthesis))
@@ -473,11 +500,6 @@ int FunctionHintProposalModel::activeArgument(const QString &prefix) const
 // -----------------------------
 // QmlJSCompletionAssistProvider
 // -----------------------------
-bool QmlJSCompletionAssistProvider::supportsEditor(Core::Id editorId) const
-{
-    return editorId == Constants::C_QMLJSEDITOR_ID;
-}
-
 int QmlJSCompletionAssistProvider::activationCharSequenceLength() const
 {
     return 1;
@@ -506,12 +528,11 @@ QmlJSCompletionAssistProcessor::QmlJSCompletionAssistProcessor()
     , m_snippetCollector(QLatin1String(Constants::QML_SNIPPETS_GROUP_ID), iconForColor(Qt::red), SnippetOrder)
 {}
 
-QmlJSCompletionAssistProcessor::~QmlJSCompletionAssistProcessor()
-{}
+QmlJSCompletionAssistProcessor::~QmlJSCompletionAssistProcessor() = default;
 
 IAssistProposal *QmlJSCompletionAssistProcessor::createContentProposal() const
 {
-    GenericProposalModel *model = new QmlJSAssistProposalModel(m_completions);
+    GenericProposalModelPtr model(new QmlJSAssistProposalModel(m_completions));
     return new GenericProposal(m_startPosition, model);
 }
 
@@ -519,10 +540,9 @@ IAssistProposal *QmlJSCompletionAssistProcessor::createHintProposal(
         const QString &functionName, const QStringList &namedArguments,
         int optionalNamedArguments, bool isVariadic) const
 {
-    IFunctionHintProposalModel *model = new FunctionHintProposalModel(
-                functionName, namedArguments, optionalNamedArguments, isVariadic);
-    IAssistProposal *proposal = new FunctionHintProposal(m_startPosition, model);
-    return proposal;
+    FunctionHintProposalModelPtr model(new FunctionHintProposalModel(
+                functionName, namedArguments, optionalNamedArguments, isVariadic));
+    return new FunctionHintProposal(m_startPosition, model);
 }
 
 IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *assistInterface)
@@ -530,7 +550,7 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
     m_interface.reset(static_cast<const QmlJSCompletionAssistInterface *>(assistInterface));
 
     if (assistInterface->reason() == IdleEditor && !acceptsIdleEditor())
-        return 0;
+        return nullptr;
 
     const QString &fileName = m_interface->fileName();
 
@@ -541,11 +561,10 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
 
     m_completions.clear();
 
-    const QmlJSCompletionAssistInterface *qmlInterface =
-            static_cast<const QmlJSCompletionAssistInterface *>(assistInterface);
+    auto qmlInterface = static_cast<const QmlJSCompletionAssistInterface *>(assistInterface);
     const SemanticInfo &semanticInfo = qmlInterface->semanticInfo();
     if (!semanticInfo.isValid())
-        return 0;
+        return nullptr;
 
     const Document::Ptr document = semanticInfo.document;
     const QFileInfo currentFileInfo(fileName);
@@ -571,7 +590,7 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
     startPositionCursor.setPosition(m_startPosition);
     CompletionContextFinder contextFinder(startPositionCursor);
 
-    const ObjectValue *qmlScopeType = 0;
+    const ObjectValue *qmlScopeType = nullptr;
     if (contextFinder.isInQmlContext()) {
         // find the enclosing qml object
         // ### this should use semanticInfo.declaringMember instead, but that may also return functions
@@ -586,13 +605,13 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
         }
         // grouped property bindings change the scope type
         for (i++; i < path.size(); ++i) {
-            AST::UiObjectDefinition *objDef = AST::cast<AST::UiObjectDefinition *>(path[i]);
+            auto objDef = AST::cast<AST::UiObjectDefinition *>(path[i]);
             if (!objDef || !document->bind()->isGroupedPropertyBinding(objDef))
                 break;
             const ObjectValue *newScopeType = qmlScopeType;
             for (AST::UiQualifiedId *it = objDef->qualifiedTypeNameId; it; it = it->next) {
                 if (!newScopeType || it->name.isEmpty()) {
-                    newScopeType = 0;
+                    newScopeType = nullptr;
                     break;
                 }
                 const Value *v = newScopeType->lookupMember(it->name.toString(), context);
@@ -622,7 +641,7 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
         if (!literalText.isEmpty()
                 && literalText.at(0) != QLatin1Char('"')
                 && literalText.at(0) != QLatin1Char('\'')) {
-            return 0;
+            return nullptr;
         }
 
         literalText = literalText.mid(1);
@@ -632,7 +651,7 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
             patterns << QLatin1String("*.qml") << QLatin1String("*.js");
             if (completeFileName(document->path(), literalText, patterns))
                 return createContentProposal();
-            return 0;
+            return nullptr;
         }
 
         const Value *value =
@@ -646,7 +665,7 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
 
         // ### enum completion?
 
-        return 0;
+        return nullptr;
     }
 
     // currently path-in-stringliteral is the only completion available in imports
@@ -675,11 +694,11 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
                             nCompletions.append(completion.right(completion.size()-toSkip));
                     completions = nCompletions;
                 }
-                addCompletions(&m_completions, completions, m_interface->fileNameIcon(), KeywordOrder);
+                addCompletions(&m_completions, completions, QmlJSCompletionAssistInterface::fileNameIcon(), KeywordOrder);
                 return createContentProposal();
             }
         }
-        return 0;
+        return nullptr;
     }
 
     // member "a.bc<complete>" or function "foo(<complete>" completion
@@ -693,7 +712,7 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
         QmlExpressionUnderCursor expressionUnderCursor;
         AST::ExpressionNode *expression = expressionUnderCursor(tc);
 
-        if (expression != 0 && ! isLiteral(expression)) {
+        if (expression && ! isLiteral(expression)) {
             // Evaluate the expression under cursor.
             ValueOwner *interp = context->valueOwner();
             const Value *value =
@@ -703,12 +722,12 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
             if (value && completionOperator == QLatin1Char('.')) { // member completion
                 ProcessProperties processProperties(&scopeChain);
                 if (contextFinder.isInLhsOfBinding() && qmlScopeType) {
-                    LhsCompletionAdder completionAdder(&m_completions, m_interface->symbolIcon(),
+                    LhsCompletionAdder completionAdder(&m_completions, QmlJSCompletionAssistInterface::symbolIcon(),
                                                        PropertyOrder, contextFinder.isAfterOnInLhsOfBinding());
                     processProperties.setEnumerateGeneratedSlots(true);
                     processProperties(value, &completionAdder);
                 } else {
-                    CompletionAdder completionAdder(&m_completions, m_interface->symbolIcon(), SymbolOrder);
+                    CompletionAdder completionAdder(&m_completions, QmlJSCompletionAssistInterface::symbolIcon(), SymbolOrder);
                     processProperties(value, &completionAdder);
                 }
             } else if (value
@@ -733,7 +752,7 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
 
         if (! m_completions.isEmpty())
             return createContentProposal();
-        return 0;
+        return nullptr;
     }
 
     // global completion
@@ -752,25 +771,25 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
             ProcessProperties processProperties(&scopeChain);
             processProperties.setGlobalCompletion(true);
             processProperties.setEnumerateGeneratedSlots(true);
-            processProperties.setEnumerateSlots(false);
+            processProperties.setEnumerateMethods(false);
 
             // id: is special
             AssistProposalItem *idProposalItem = new QmlJSAssistProposalItem;
             idProposalItem->setText(QLatin1String("id: "));
-            idProposalItem->setIcon(m_interface->symbolIcon());
+            idProposalItem->setIcon(QmlJSCompletionAssistInterface::symbolIcon());
             idProposalItem->setOrder(PropertyOrder);
             m_completions.append(idProposalItem);
 
             {
-                LhsCompletionAdder completionAdder(&m_completions, m_interface->symbolIcon(),
+                LhsCompletionAdder completionAdder(&m_completions, QmlJSCompletionAssistInterface::symbolIcon(),
                                                    PropertyOrder, contextFinder.isAfterOnInLhsOfBinding());
                 processProperties(qmlScopeType, &completionAdder);
             }
 
             if (ScopeBuilder::isPropertyChangesObject(context, qmlScopeType)
                     && scopeChain.qmlScopeObjects().size() == 2) {
-                CompletionAdder completionAdder(&m_completions, m_interface->symbolIcon(), SymbolOrder);
-                processProperties(scopeChain.qmlScopeObjects().first(), &completionAdder);
+                CompletionAdder completionAdder(&m_completions, QmlJSCompletionAssistInterface::symbolIcon(), SymbolOrder);
+                processProperties(scopeChain.qmlScopeObjects().constFirst(), &completionAdder);
             }
         }
 
@@ -789,7 +808,7 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
                         completion = QString::fromLatin1("\"%1\"").arg(key);
                     else
                         completion = QString::fromLatin1("%1.%2").arg(name, key);
-                    addCompletion(&m_completions, key, m_interface->symbolIcon(),
+                    addCompletion(&m_completions, key, QmlJSCompletionAssistInterface::symbolIcon(),
                                   EnumValueOrder, completion);
                 }
             }
@@ -801,7 +820,7 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
         if (doQmlTypeCompletion) {
             if (const ObjectValue *qmlTypes = scopeChain.qmlTypes()) {
                 ProcessProperties processProperties(&scopeChain);
-                CompletionAdder completionAdder(&m_completions, m_interface->symbolIcon(), TypeOrder);
+                CompletionAdder completionAdder(&m_completions, QmlJSCompletionAssistInterface::symbolIcon(), TypeOrder);
                 processProperties(qmlTypes, &completionAdder);
             }
         }
@@ -810,42 +829,40 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
             // It's a global completion.
             ProcessProperties processProperties(&scopeChain);
             processProperties.setGlobalCompletion(true);
-            CompletionAdder completionAdder(&m_completions, m_interface->symbolIcon(), SymbolOrder);
+            CompletionAdder completionAdder(&m_completions, QmlJSCompletionAssistInterface::symbolIcon(), SymbolOrder);
             processProperties(&completionAdder);
         }
 
         if (doJsKeywordCompletion) {
             // add js keywords
-            addCompletions(&m_completions, Scanner::keywords(), m_interface->keywordIcon(), KeywordOrder);
+            addCompletions(&m_completions, Scanner::keywords(), QmlJSCompletionAssistInterface::keywordIcon(), KeywordOrder);
         }
 
         // add qml extra words
         if (doQmlKeywordCompletion && isQmlFile) {
-            static QStringList qmlWords;
-            static QStringList qmlWordsAlsoInJs;
+            static QStringList qmlWords{
+                QLatin1String("property"),
+                //QLatin1String("readonly")
+                QLatin1String("signal"),
+                QLatin1String("import")
+            };
+            static QStringList qmlWordsAlsoInJs{
+                QLatin1String("default"), QLatin1String("function")
+            };
 
-            if (qmlWords.isEmpty()) {
-                qmlWords << QLatin1String("property")
-                            //<< QLatin1String("readonly")
-                         << QLatin1String("signal")
-                         << QLatin1String("import");
-            }
-            if (qmlWordsAlsoInJs.isEmpty())
-                qmlWordsAlsoInJs << QLatin1String("default") << QLatin1String("function");
-
-            addCompletions(&m_completions, qmlWords, m_interface->keywordIcon(), KeywordOrder);
+            addCompletions(&m_completions, qmlWords, QmlJSCompletionAssistInterface::keywordIcon(), KeywordOrder);
             if (!doJsKeywordCompletion)
-                addCompletions(&m_completions, qmlWordsAlsoInJs, m_interface->keywordIcon(), KeywordOrder);
+                addCompletions(&m_completions, qmlWordsAlsoInJs, QmlJSCompletionAssistInterface::keywordIcon(), KeywordOrder);
         }
 
         m_completions.append(m_snippetCollector.collect());
 
         if (! m_completions.isEmpty())
             return createContentProposal();
-        return 0;
+        return nullptr;
     }
 
-    return 0;
+    return nullptr;
 }
 
 bool QmlJSCompletionAssistProcessor::acceptsIdleEditor() const
@@ -858,6 +875,8 @@ bool QmlJSCompletionAssistProcessor::acceptsIdleEditor() const
         maybeAccept = true;
     } else {
         const QChar &charUnderCursor = m_interface->textDocument()->characterAt(cursorPos);
+        if (isValidIdentifierChar(charUnderCursor))
+            return false;
         if (isIdentifierChar(charBeforeCursor)
                 && ((charUnderCursor.isSpace()
                     || charUnderCursor.isNull()
@@ -930,7 +949,7 @@ bool QmlJSCompletionAssistProcessor::completeFileName(const QString &relativeBas
 
         AssistProposalItem *item = new QmlJSAssistProposalItem;
         item->setText(fileName);
-        item->setIcon(m_interface->fileNameIcon());
+        item->setIcon(QmlJSCompletionAssistInterface::fileNameIcon());
         m_completions.append(item);
     }
 
@@ -968,9 +987,6 @@ QmlJSCompletionAssistInterface::QmlJSCompletionAssistInterface(QTextDocument *te
                                                                const SemanticInfo &info)
     : AssistInterface(textDocument, position, fileName, reason)
     , m_semanticInfo(info)
-    , m_darkBlueIcon(iconForColor(Qt::darkBlue))
-    , m_darkYellowIcon(iconForColor(Qt::darkYellow))
-    , m_darkCyanIcon(iconForColor(Qt::darkCyan))
 {}
 
 const SemanticInfo &QmlJSCompletionAssistInterface::semanticInfo() const
@@ -978,6 +994,24 @@ const SemanticInfo &QmlJSCompletionAssistInterface::semanticInfo() const
     return m_semanticInfo;
 }
 
+const QIcon &QmlJSCompletionAssistInterface::fileNameIcon()
+{
+    static QIcon darkBlueIcon(iconForColor(Qt::darkBlue));
+    return darkBlueIcon;
+}
+
+const QIcon &QmlJSCompletionAssistInterface::keywordIcon()
+{
+    static QIcon darkYellowIcon(iconForColor(Qt::darkYellow));
+    return darkYellowIcon;
+}
+
+const QIcon &QmlJSCompletionAssistInterface::symbolIcon()
+{
+    static QIcon darkCyanIcon(iconForColor(Qt::darkCyan));
+    return darkCyanIcon;
+}
+
 namespace {
 
 class QmlJSLessThan
@@ -985,7 +1019,7 @@ class QmlJSLessThan
 public:
     QmlJSLessThan(const QString &searchString) : m_searchString(searchString)
     { }
-    bool operator() (const AssistProposalItem *a, const AssistProposalItem *b)
+    bool operator() (const AssistProposalItemInterface *a, const AssistProposalItemInterface *b)
     {
         if (a->order() != b->order())
             return a->order() > b->order();
@@ -993,8 +1027,6 @@ public:
             return true;
         else if (b->text().isEmpty())
             return false;
-        else if (a->data().isValid() != b->data().isValid())
-            return a->data().isValid();
         else if (a->text().at(0).isUpper() && b->text().at(0).isLower())
             return false;
         else if (a->text().at(0).isLower() && b->text().at(0).isUpper())
@@ -1019,9 +1051,9 @@ void QmlJSAssistProposalModel::filter(const QString &prefix)
     GenericProposalModel::filter(prefix);
     if (prefix.startsWith(QLatin1String("__")))
         return;
-    QList<AssistProposalItem *> newCurrentItems;
+    QList<AssistProposalItemInterface *> newCurrentItems;
     newCurrentItems.reserve(m_currentItems.size());
-    foreach (AssistProposalItem *item, m_currentItems)
+    foreach (AssistProposalItemInterface *item, m_currentItems)
         if (!item->text().startsWith(QLatin1String("__")))
             newCurrentItems << item;
     m_currentItems = newCurrentItems;
