@@ -513,6 +513,7 @@ void MainWidget::addStateView(BaseItem *item)
     });
     connect(view->view(), &GraphicsView::panningChanged, m_actionHandler->action(ActionPan), &QAction::setChecked);
     connect(view->view(), &GraphicsView::magnifierChanged, m_actionHandler->action(ActionMagnifier), &QAction::setChecked);
+    connect(m_magnifier, &Magnifier::visibilityChanged, m_actionHandler->action(ActionMagnifier), &QAction::setChecked);
     connect(view->scene(), &GraphicsScene::openStateView, this, &MainWidget::addStateView, Qt::QueuedConnection);
     connect(view->scene(), &GraphicsScene::selectedStateCountChanged, this, [this](int count) {
         bool currentView = sender() == m_views.last()->scene();
@@ -535,7 +536,7 @@ void MainWidget::addStateView(BaseItem *item)
         m_actionHandler->action(ActionPaste)->setEnabled(currentView && para);
     });
 
-    if (m_views.count() > 0)
+    if (!m_views.isEmpty())
         m_views.last()->scene()->unselectAll();
 
     if (item) {
@@ -582,7 +583,7 @@ void MainWidget::newDocument()
 void MainWidget::clear()
 {
     // Clear and delete all stateviews
-    while (m_views.count() > 0) {
+    while (!m_views.isEmpty()) {
         m_views.last()->clear();
         delete m_views.takeLast();
     }
