@@ -291,7 +291,9 @@ void SerialOutputPane::createNewOutputWindow(SerialControl *rc)
 
     connect(rc, &SerialControl::finished,
             [this, rc]() {
-        rc->outputFormatter()->flush();
+        const int tabIndex = indexOf(rc);
+        if (tabIndex != -1)
+            m_serialControlTabs[tabIndex].window->flush();
         if (isCurrent(rc))
             enableButtons(rc, false);
     });
@@ -299,11 +301,9 @@ void SerialOutputPane::createNewOutputWindow(SerialControl *rc)
     connect(rc, &SerialControl::appendMessageRequested,
             this, &SerialOutputPane::appendMessage);
 
-    Utils::OutputFormatter *formatter = rc->outputFormatter();
-
     // Create new
     static int counter = 0;
-    Core::Id contextId = Core::Id(Constants::C_SERIAL_OUTPUT).withSuffix(counter++);
+    Utils::Id contextId = Utils::Id(Constants::C_SERIAL_OUTPUT).withSuffix(counter++);
     Core::Context context(contextId);
     auto ow = new Core::OutputWindow(context, QString(), m_tabWidget);
     using TextEditor::TextEditorSettings;
@@ -315,7 +315,6 @@ void SerialOutputPane::createNewOutputWindow(SerialControl *rc)
             this, fontSettingsChanged);
     fontSettingsChanged();
     ow->setWindowTitle(tr("Serial Terminal Window"));
-    ow->setFormatter(formatter);
     // TODO: wordwrap, maxLineCount, zoom/wheelZoom (add to settings)
 
     auto controlTab = SerialControlTab(rc, ow);
@@ -367,7 +366,7 @@ void SerialOutputPane::createToolButtons()
 
     // Reset button
     m_resetButton = new QToolButton;
-    m_resetButton->setIcon(Utils::Icons::RELOAD.icon());
+    m_resetButton->setIcon(Utils::Icons::RELOAD_TOOLBAR.icon());
     m_resetButton->setToolTip(tr("Reset Board"));
     m_resetButton->setEnabled(false);
 
