@@ -38,10 +38,12 @@
 
 using namespace TextEditor;
 
+QT_BEGIN_NAMESPACE
 uint qHash(const AssistProposalItem &item)
 {
     return qHash(item.text());
 }
+QT_END_NAMESPACE
 
 namespace {
 
@@ -114,12 +116,11 @@ struct ContentLessThan
     {
         bool operator()(const QChar &a, const QChar &b)
         {
-            if (a == QLatin1Char('_'))
+            if (a == '_')
                 return false;
-            else if (b == QLatin1Char('_'))
+            if (b == '_')
                 return true;
-            else
-                return a < b;
+            return a < b;
         }
     };
 
@@ -129,8 +130,7 @@ private:
 
 } // Anonymous
 
-GenericProposalModel::GenericProposalModel()
-{}
+GenericProposalModel::GenericProposalModel() = default;
 
 GenericProposalModel::~GenericProposalModel()
 {
@@ -157,7 +157,7 @@ static QString cleanText(const QString &original)
     int ignore = 0;
     for (int i = clean.length() - 1; i >= 0; --i, ++ignore) {
         const QChar &c = clean.at(i);
-        if (c.isLetterOrNumber() || c == QLatin1Char('_')
+        if (c.isLetterOrNumber() || c == '_'
                 || c.isHighSurrogate() || c.isLowSurrogate()) {
             break;
         }
@@ -167,29 +167,61 @@ static QString cleanText(const QString &original)
     return clean;
 }
 
+static bool textStartsWith(CaseSensitivity cs, const QString &text, const QString &prefix)
+{
+    switch (cs) {
+    case TextEditor::CaseInsensitive:
+        return text.startsWith(prefix, Qt::CaseInsensitive);
+    case TextEditor::CaseSensitive:
+        return text.startsWith(prefix, Qt::CaseSensitive);
+    case TextEditor::FirstLetterCaseSensitive:
+        return prefix.at(0) == text.at(0)
+               && prefix.midRef(1).startsWith(text.midRef(1), Qt::CaseInsensitive);
+    }
+
+    return false;
+}
+
+enum class PerfectMatchType {
+    None,
+    StartsWith,
+    Full,
+};
+
+static PerfectMatchType perfectMatch(CaseSensitivity cs, const QString &text, const QString &prefix)
+{
+    if (textStartsWith(cs, text, prefix))
+        return prefix.size() == text.size() ? PerfectMatchType::Full : PerfectMatchType::StartsWith;
+
+    return PerfectMatchType::None;
+}
+
 bool GenericProposalModel::isPerfectMatch(const QString &prefix) const
 {
     if (prefix.isEmpty())
         return false;
 
+    const CaseSensitivity cs = TextEditorSettings::completionSettings().m_caseSensitivity;
+    bool hasFullMatch = false;
+
     for (int i = 0; i < size(); ++i) {
         const QString &current = cleanText(text(i));
-        if (!current.isEmpty()) {
-            CaseSensitivity cs = TextEditorSettings::completionSettings().m_caseSensitivity;
-            if (cs == TextEditor::CaseSensitive) {
-                if (prefix == current)
-                    return true;
-            } else if (cs == TextEditor::CaseInsensitive) {
-                if (prefix.compare(current, Qt::CaseInsensitive) == 0)
-                    return true;
-            } else if (cs == TextEditor::FirstLetterCaseSensitive) {
-                if (prefix.at(0) == current.at(0)
-                        && prefix.midRef(1).compare(current.midRef(1), Qt::CaseInsensitive) == 0)
-                    return true;
-            }
+        if (current.isEmpty())
+            continue;
+
+        const PerfectMatchType match = perfectMatch(cs, current, prefix);
+        if (match == PerfectMatchType::StartsWith)
+            return false;
+
+        if (match == PerfectMatchType::Full) {
+            if (proposalItem(i)->isKeyword())
+                return true;
+
+            hasFullMatch = true;
         }
     }
-    return false;
+
+    return hasFullMatch;
 }
 
 bool GenericProposalModel::isPrefiltered(const QString &prefix) const
@@ -228,6 +260,11 @@ QString GenericProposalModel::detail(int index) const
     return m_currentItems.at(index)->detail();
 }
 
+Qt::TextFormat GenericProposalModel::detailFormat(int index) const
+{
+    return m_currentItems.at(index)->detailFormat();
+}
+
 void GenericProposalModel::removeDuplicates()
 {
     if (m_duplicatesRemoved)
@@ -255,46 +292,51 @@ void GenericProposalModel::filter(const QString &prefix)
     if (prefix.isEmpty())
         return;
 
-    const CamelHumpMatcher::CaseSensitivity caseSensitivity =
+    const FuzzyMatcher::CaseSensitivity caseSensitivity =
         convertCaseSensitivity(TextEditorSettings::completionSettings().m_caseSensitivity);
-    const QRegularExpression regExp = CamelHumpMatcher::createCamelHumpRegExp(prefix, caseSensitivity);
+    const QRegularExpression regExp = FuzzyMatcher::createRegExp(prefix, caseSensitivity);
 
     m_currentItems.clear();
     const QString lowerPrefix = prefix.toLower();
-    foreach (const auto &item, m_originalItems) {
+    for (const auto &item : qAsConst(m_originalItems)) {
         const QString &text = item->text();
-        if (regExp.match(text).capturedStart() == 0) {
+        const QRegularExpressionMatch match = regExp.match(text);
+        const bool hasPrefixMatch = match.capturedStart() == 0;
+        const bool hasInfixMatch = prefix.size() >= 3 && match.hasMatch();
+        if (hasPrefixMatch || hasInfixMatch) {
             m_currentItems.append(item);
             if (text.startsWith(prefix)) {
                 // Direct match
-                item->setPrefixMatch(text.length() == prefix.length()
-                                     ? AssistProposalItemInterface::PrefixMatch::Full
-                                     : AssistProposalItemInterface::PrefixMatch::Exact);
+                item->setProposalMatch(text.length() == prefix.length()
+                                       ? AssistProposalItemInterface::ProposalMatch::Full
+                                       : AssistProposalItemInterface::ProposalMatch::Exact);
                 continue;
             }
 
             if (text.startsWith(lowerPrefix, Qt::CaseInsensitive))
-                item->setPrefixMatch(AssistProposalItemInterface::PrefixMatch::Lower);
+                item->setProposalMatch(AssistProposalItemInterface::ProposalMatch::Prefix);
+            else if (text.contains(lowerPrefix, Qt::CaseInsensitive))
+                item->setProposalMatch(AssistProposalItemInterface::ProposalMatch::Infix);
         }
     }
 }
 
-CamelHumpMatcher::CaseSensitivity
+FuzzyMatcher::CaseSensitivity
     GenericProposalModel::convertCaseSensitivity(TextEditor::CaseSensitivity textEditorCaseSensitivity)
 {
     switch (textEditorCaseSensitivity) {
     case TextEditor::CaseSensitive:
-        return CamelHumpMatcher::CaseSensitivity::CaseSensitive;
+        return FuzzyMatcher::CaseSensitivity::CaseSensitive;
     case TextEditor::FirstLetterCaseSensitive:
-        return CamelHumpMatcher::CaseSensitivity::FirstLetterCaseSensitive;
+        return FuzzyMatcher::CaseSensitivity::FirstLetterCaseSensitive;
     default:
-        return CamelHumpMatcher::CaseSensitivity::CaseInsensitive;
+        return FuzzyMatcher::CaseSensitivity::CaseInsensitive;
     }
 }
 
 bool GenericProposalModel::isSortable(const QString &prefix) const
 {
-    Q_UNUSED(prefix);
+    Q_UNUSED(prefix)
 
     if (m_currentItems.size() < kMaxSort)
         return true;
