@@ -40,7 +40,7 @@
 #include <qfile.h>
 #include <qfileinfo.h>
 #include <qlist.h>
-#include <qregexp.h>
+#include <qregularexpression.h>
 #include <qset.h>
 #include <qstringlist.h>
 #include <qtextstream.h>
@@ -459,8 +459,18 @@ void QMakeEvaluator::runProcess(QProcess *proc, const QString &command) const
 {
     proc->setWorkingDirectory(currentDirectory());
 # ifdef PROEVALUATOR_SETENV
-    if (!m_option->environment.isEmpty())
-        proc->setProcessEnvironment(m_option->environment);
+    if (!m_option->environment.isEmpty()) {
+        QProcessEnvironment env = m_option->environment;
+        static const QString dummyVar = "__qtc_dummy";
+        static const QString notSetValue = "not set";
+        const QString oldValue = env.value(dummyVar, notSetValue); // Just in case.
+        env.insert(dummyVar, "QTCREATORBUG-23504"); // Force detach.
+        if (oldValue == notSetValue)
+            env.remove(dummyVar);
+        else
+            env.insert(dummyVar, oldValue);
+        proc->setProcessEnvironment(env);
+    }
 # endif
 # ifdef Q_OS_WIN
     proc->setNativeArguments(QLatin1String("/v:off /s /c \"") + command + QLatin1Char('"'));
@@ -586,7 +596,11 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
         if (!var.isEmpty()) {
             const auto strings = values(map(var));
             if (regexp) {
-                QRegExp sepRx(sep);
+                QRegularExpression sepRx(sep, QRegularExpression::DotMatchesEverythingOption);
+                if (!sepRx.isValid()) {
+                    evalError(fL1S("section(): Encountered invalid regular expression '%1'.").arg(sep));
+                    goto allfail;
+                }
                 for (const ProString &str : strings) {
                     const QString &rstr = str.toQString(m_tmp[m_toggle ^= 1]).section(sepRx, beg, end);
                     ret << (rstr.isSharedWith(m_tmp[m_toggle]) ? str : ProString(rstr).setSource(str));
@@ -888,10 +902,14 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
         if (args.count() != 2) {
             evalError(fL1S("find(var, str) requires two arguments."));
         } else {
-            QRegExp regx(args.at(1).toQString());
+            QRegularExpression regx(args.at(1).toQString(), QRegularExpression::DotMatchesEverythingOption);
+            if (!regx.isValid()) {
+                evalError(fL1S("find(): Encountered invalid regular expression '%1'.").arg(regx.pattern()));
+                goto allfail;
+            }
             const auto vals = values(map(args.at(0)));
             for (const ProString &val : vals) {
-                if (regx.indexIn(val.toQString(m_tmp[m_toggle ^= 1])) != -1)
+                if (val.toQString(m_tmp[m_toggle ^= 1]).contains(regx))
                     ret += val;
             }
         }
@@ -1001,7 +1019,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
         break;
     case E_RE_ESCAPE:
         for (int i = 0; i < args.size(); ++i) {
-            const QString &rstr = QRegExp::escape(args.at(i).toQString(m_tmp1));
+            const QString &rstr = QRegularExpression::escape(args.at(i).toQString(m_tmp1));
             ret << (rstr.isSharedWith(m_tmp1) ? args.at(i) : ProString(rstr).setSource(args.at(i)));
         }
         break;
@@ -1054,8 +1072,12 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
                 dirs.append(QString());
             }
 
-            r.detach(); // Keep m_tmp out of QRegExp's cache
-            QRegExp regex(r, Qt::CaseSensitive, QRegExp::Wildcard);
+            QString pattern = QRegularExpression::wildcardToRegularExpression(r);
+            QRegularExpression regex(pattern, QRegularExpression::DotMatchesEverythingOption);
+            if (!regex.isValid()) {
+                evalError(fL1S("section(): Encountered invalid wildcard expression '%1'.").arg(pattern));
+                goto allfail;
+            }
             for (int d = 0; d < dirs.count(); d++) {
                 QString dir = dirs[d];
                 QDir qdir(pfx + dir);
@@ -1067,7 +1089,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
                         if (recursive)
                             dirs.append(fname + QLatin1Char('/'));
                     }
-                    if (regex.exactMatch(qdir[i]))
+                    if (regex.match(qdir[i]).hasMatch())
                         ret += ProString(fname).setSource(currentFileId());
                 }
             }
@@ -1109,7 +1131,11 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
         if (args.count() != 3 ) {
             evalError(fL1S("replace(var, before, after) requires three arguments."));
         } else {
-            const QRegExp before(args.at(1).toQString());
+            const QRegularExpression before(args.at(1).toQString(), QRegularExpression::DotMatchesEverythingOption);
+            if (!before.isValid()) {
+                evalError(fL1S("replace(): Encountered invalid regular expression '%1'.").arg(before.pattern()));
+                goto allfail;
+            }
             const QString &after(args.at(2).toQString(m_tmp2));
             const auto vals = values(map(args.at(0)));
             for (const ProString &val : vals) {
@@ -1309,6 +1335,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
         break;
     }
 
+  allfail:
     return ReturnTrue;
 }
 
@@ -1347,15 +1374,15 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
             return ReturnFalse;
         }
         const ProKey &var = map(args.at(0));
-        for (ProValueMapStack::Iterator vmi = m_valuemapStack.end();
+        for (ProValueMapStack::iterator vmi = m_valuemapStack.end();
              --vmi != m_valuemapStack.begin(); ) {
             ProValueMap::Iterator it = (*vmi).find(var);
             if (it != (*vmi).end()) {
                 if (it->constBegin() == statics.fakeValue.constBegin()) {
                     // This is stupid, but qmake doesn't propagate deletions
-                    m_valuemapStack.first()[var] = ProStringList();
+                    m_valuemapStack.front()[var] = ProStringList();
                 } else {
-                    m_valuemapStack.first()[var] = *it;
+                    m_valuemapStack.front()[var] = *it;
                 }
                 (*vmi).erase(it);
                 while (--vmi != m_valuemapStack.begin())
@@ -1370,7 +1397,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
             evalError(fL1S("discard_from(file) requires one argument."));
             return ReturnFalse;
         }
-        if (m_valuemapStack.count() != 1) {
+        if (m_valuemapStack.size() != 1) {
             evalError(fL1S("discard_from() cannot be called from functions."));
             return ReturnFalse;
         }
@@ -1379,7 +1406,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
         int pro = m_vfs->idForFileName(fn, flags | QMakeVfs::VfsAccessedOnly);
         if (!pro)
             return ReturnFalse;
-        ProValueMap &vmap = m_valuemapStack.first();
+        ProValueMap &vmap = m_valuemapStack.front();
         for (auto vit = vmap.begin(); vit != vmap.end(); ) {
             if (!vit->isEmpty()) {
                 auto isFrom = [pro](const ProString &s) {
@@ -1407,7 +1434,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
             else
                 ++fit;
         }
-        ProStringList &iif = m_valuemapStack.first()[ProKey("QMAKE_INTERNAL_INCLUDED_FILES")];
+        ProStringList &iif = m_valuemapStack.front()[ProKey("QMAKE_INTERNAL_INCLUDED_FILES")];
         int idx = iif.indexOf(ProString(fn));
         if (idx >= 0)
             iif.removeAt(idx);
@@ -1425,16 +1452,20 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
                 return ok;
             if (args.count() == 2)
                 return returnBool(vars.contains(map(args.at(1))));
-            QRegExp regx;
+            QRegularExpression regx;
             const QString &qry = args.at(2).toQString(m_tmp1);
-            if (qry != QRegExp::escape(qry)) {
-                QString copy = qry;
-                copy.detach();
-                regx.setPattern(copy);
+            if (qry != QRegularExpression::escape(qry)) {
+                regx.setPattern(QRegularExpression::anchoredPattern(qry));
+                if (!regx.isValid()) {
+                    evalError(fL1S("infile(): Encountered invalid regular expression '%1'.").arg(qry));
+                    return ReturnFalse;
+                }
             }
             const auto strings = vars.value(map(args.at(1)));
             for (const ProString &s : strings) {
-                if ((!regx.isEmpty() && regx.exactMatch(s.toQString(m_tmp[m_toggle ^= 1]))) || s == qry)
+                if ((!regx.pattern().isEmpty()
+                             && regx.match(s.toQString(m_tmp[m_toggle ^= 1])).hasMatch())
+                        || s == qry)
                     return ReturnTrue;
             }
         }
@@ -1492,17 +1523,22 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
         }
 
         const QString &qry = args.at(1).toQString(m_tmp1);
-        QRegExp regx;
-        if (qry != QRegExp::escape(qry)) {
-            QString copy = qry;
-            copy.detach();
-            regx.setPattern(copy);
+        QRegularExpression regx;
+        regx.setPatternOptions(QRegularExpression::DotMatchesEverythingOption);
+        if (qry != QRegularExpression::escape(qry)) {
+            regx.setPattern(QRegularExpression::anchoredPattern(qry));
+            if (!regx.isValid()) {
+                evalError(fL1S("contains(): Encountered invalid regular expression '%1'.").arg(qry));
+                return ReturnFalse;
+            }
         }
+
         const ProStringList &l = values(map(args.at(0)));
         if (args.count() == 2) {
             for (int i = 0; i < l.size(); ++i) {
                 const ProString &val = l[i];
-                if ((!regx.isEmpty() && regx.exactMatch(val.toQString(m_tmp[m_toggle ^= 1]))) || val == qry)
+                if ((!regx.pattern().isEmpty() && regx.match(val.toQString(m_tmp[m_toggle ^= 1])).hasMatch())
+                        || val == qry)
                     return ReturnTrue;
             }
         } else {
@@ -1511,8 +1547,8 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
                 const ProString val = l[i];
                 for (int mut = 0; mut < mutuals.count(); mut++) {
                     if (val.toQStringRef() == mutuals[mut].trimmed()) {
-                        return returnBool((!regx.isEmpty()
-                                           && regx.exactMatch(val.toQString(m_tmp[m_toggle ^= 1])))
+                        return returnBool((!regx.pattern().isEmpty()
+                                           && regx.match(val.toQString(m_tmp[m_toggle ^= 1])).hasMatch())
                                           || val == qry);
                     }
                 }
@@ -1643,7 +1679,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
             return ReturnFalse;
         }
         QString parseInto;
-        LoadFlags flags = 0;
+        LoadFlags flags;
         if (m_cumulative)
             flags = LoadSilent;
         if (args.count() >= 2) {
@@ -1742,7 +1778,6 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
             evalError(fL1S("system(exec) requires one argument."));
             return ReturnFalse;
         }
-#ifdef PROEVALUATOR_FULL
         if (m_cumulative) // Anything else would be insanity
             return ReturnFalse;
 #ifndef QT_BOOTSTRAPPED
@@ -1759,9 +1794,6 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
             raise(WTERMSIG(ec));
 #  endif
         return returnBool(ec == 0);
-#endif
-#else
-        return ReturnTrue;
 #endif
     }
     case T_ISEMPTY: {
